#include <ostream>
#include <vector>
#include <unordered_map>
#include "stl_printing.hpp"
using namespace std;

/* Suroviny, ktore mozete prevazat. */
enum class Materials : int {
    IronOre,
    Coal,
    Logs,
    Grain,
    Steel,
    Planks,
    Food,
    Goods
};

inline ostream& operator<<(ostream& out, const Materials &mat) {
    if (mat == Materials::IronOre) {
        out << "IronOre";
    } else if (mat == Materials::Coal) {
        out << "Coal";
    } else if (mat == Materials::Logs) {
        out << "Logs";
    } else if (mat == Materials::Grain) {
        out << "Grain";
    } else if (mat == Materials::Steel) {
        out << "Steel";
    } else if (mat == Materials::Planks) {
        out << "Planks";
    } else if (mat == Materials::Food) {
        out << "Food";
    } else if (mat == Materials::Goods) {
        out << "Goods";
    } else {
        out << "Unknown material";
    }
    return out;
}

enum class FactoryType {
    Forest,
    Farm,
    CoalMine,
    IronMine,
    SteelMill,
    SawMill,
    GoodsFactory,
    FoodFactory,
    Town
};

inline ostream& operator<<(ostream& out, const FactoryType &fact) {
    if (fact == FactoryType::IronMine) {
        out << "IronMine";
    } else if (fact == FactoryType::CoalMine) {
        out << "CoalMine";
    } else if (fact == FactoryType::Forest) {
        out << "Forest";
    } else if (fact == FactoryType::Farm) {
        out << "Farm";
    } else if (fact == FactoryType::FoodFactory) {
        out << "FoodFactory";
    } else if (fact == FactoryType::GoodsFactory) {
        out << "GoodsFactory";
    } else if (fact == FactoryType::Town) {
        out << "Town";
    } else {
        out << "Other factory type";
    }
    return out;
}

/* Táto trieda reprezentuje smery */
enum class Directions : int {
    DirectionNE,
    DirectionE,
    DirectionSE,
    DirectionSW,
    DirectionW,
    DirectionNW,
    DirectionNONE = -1
};

inline ostream& operator<<(ostream& out, const Directions &dir) {
    if (dir == Directions::DirectionNE) {
        out << "NE";
    } else if (dir == Directions::DirectionE) {
        out << "E";
    } else if (dir == Directions::DirectionSE) {
        out << "SE";
    } else if (dir == Directions::DirectionSW) {
        out << "SW";
    } else if (dir == Directions::DirectionW) {
        out << "W";
    } else if (dir == Directions::DirectionNW) {
        out << "NW";
    } else {
        out << "NONE";
    }
    return out;
}

/* Recept je trieda, ktorá reprezentuje výrobný proces suroviny. Má vstupné a výstupné materiály. */
struct Recipe {
    vector<Materials> input;
    vector<Materials> output;

    Recipe() = default;

    Recipe(vector<Materials> input, vector<Materials> output) { 
        this->input = input;
        this->output = output;
    }

    static Recipe get_factory_recipe(FactoryType factory) {
        unordered_map<FactoryType, Recipe> recipes = {
            { FactoryType::IronMine, Recipe({}, {Materials::IronOre}) },
            { FactoryType::CoalMine, Recipe({}, {Materials::Coal}) },
            { FactoryType::Forest, Recipe({}, {Materials::Logs}) },
            { FactoryType::Farm, Recipe({}, {Materials::Grain}) },
            { FactoryType::SteelMill, Recipe({Materials::IronOre, Materials::Coal}, {Materials::Steel}) },
            { FactoryType::SawMill, Recipe({Materials::Logs}, {Materials::Planks}) },
            { FactoryType::FoodFactory, Recipe({Materials::Grain}, {Materials::Food}) },
            { FactoryType::GoodsFactory, Recipe({Materials::Steel, Materials::Planks}, {Materials::Goods}) },
            { FactoryType::Town, Recipe({Materials::Goods, Materials::Food}, {}) }
        };
        return recipes.at(factory);
    }

    friend std::ostream& operator<<(std::ostream& os, const Recipe& recipe) {
        os << "Recipe(" << recipe.input << ", " << recipe.output << ")";
        return os;
    }
};
